KTMCPUManager = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDebug-2.0")

function KTMCPUManager:OnEnable()
	self:RegisterEvent("UNIT_TARGET")
	self:RegisterEvent("UNIT_COMBAT")
	self.hasHostileTarget = false
	self.inCombat = false
	self:Print("KTMCPUManager enabled")
	
	-- In case we get enabled in the middle of combat, we should turn KTM on
	self:UpdateKTMEnabled(true, InCombatLockdown())
end

function KTMCPUManager:OnDisable()
	self.hasHostileTarget = false
	self.inCombat = false
	
	-- Turn KTM back on
	self:UpdateKTMEnabled(true, true)		
end

function KTMCPUManager:UpdateKTMEnabled(force, val)
	local f = ""
	if force then
		val = val or false
		klhtm.isenabled = val
		f = " (forced)"
	else
		klhtm.isenabled = (self.hasHostileTarget or self.inCombat) or false
	end
	self:Debug("Setting KTM-Enabled to " .. tostring(klhtm.isenabled) .. f)
end

-- Any interesting unit that participates in combat should turn on KTM. We'll turn it off after 15 seconds of no other activity.
function KTMCPUManager:DisableCombatFlag()
	self.inCombat = false
	self:UpdateKTMEnabled()
end

-- Is the player solo without a pet?
function KTMCPUManager:IsSolo(arg1)	
	if UnitIsUnit(arg1, "player") and (GetNumPartyMembers() + GetNumRaidMembers() == 0) and not UnitExists("playerpet") then
		return true
	end
	return false
end

-- Any time an interesting unit affects combat, we should turn on KTM.
function KTMCPUManager:UNIT_COMBAT()
	-- We don't care about non-friendlies in combat
	if UnitIsEnemy("player", arg1) then return end
	
	-- If we're solo, we don't care about it
	if self:IsSolo(arg1) then
		self.inCombat = false
		self:UpdateKTMEnabled(true, false)
		return
	end

	-- If the unit is us, or in our group, then enable combat flag and set disable timeout to 15 sec
	if UnitIsUnit(arg1, "player") or UnitIsUnit(arg1, "playerpet") or UnitPlayerOrPetInParty(arg1) or UnitPlayerOrPetInRaid(arg1) then	
		self.inCombat = true
		self:UpdateKTMEnabled()

		-- Disable after 15 sec of no activity
		self:ScheduleEvent("disable_combat_flag", KTMCPUManager.DisableCombatFlag, 15, self)
	end
end

-- Any time we or a memeber of our party/raid selects a hostile target, we'll turn on KTM in anticipation of combat
function KTMCPUManager:UNIT_TARGET()
	-- If we're already in combat, we'll just skip this.
	if(self.inCombat) then return end
	
	-- npctarget is an invalid unit
	if arg1 == "npc" then return end
	
	if self:IsSolo(arg1) then 
		self.hasHostileTarget = false
		self:UpdateKTMEnabled(true, false)
		return
	end
	
	local t = arg1 .. "target"
	self.hasHostileTarget = false
	if UnitExists(t) and UnitIsEnemy("player", t) and not UnitIsDead(t) and not UnitPlayerControlled(t) then
		self.hasHostileTarget = true
	elseif not UnitExists(arg1 .. "target") then
		if(UnitInRaid("player")) then
			for i = 1, GetNumRaidMembers() do
				local t = "raid" .. i .. "target"
				if UnitExists(t) and UnitIsEnemy("player", t) and not UnitIsDead(t) and not UnitPlayerControlled(t) then
					self.hasHostileTarget = true
					break
				end
			end
		elseif(UnitInParty("player")) then
			for i = 1, GetNumPartyMembers() do
				local t = "party" .. i .. "target"
				if UnitExists(t) and UnitIsEnemy("player", t) and not UnitIsDead(t) and not UnitPlayerControlled(t) then
					self.hasHostileTarget = true
					break
				end
			end
		end
	end
	self:UpdateKTMEnabled()
end

KTMCPUManager:RegisterChatCommand({"/ktmcpu"})